package com.webproglabs.lab1.lab34.controller;

import com.webproglabs.lab1.WebConfiguration;
import com.webproglabs.lab1.lab34.model.Comment;
import com.webproglabs.lab1.lab34.model.Profile;
import com.webproglabs.lab1.lab34.services.CommentService;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/comment")
public class CommentController {
    private final CommentService commentService;

    public CommentController(CommentService commentService) {
        this.commentService = commentService;
    }

    @GetMapping("/{id}")
    public CommentDto getComment(@PathVariable Long id) {
        return new CommentDto(commentService.findComment(id));
    }

    @GetMapping
    public List<CommentDto> getComments() {
        return commentService.findAllComments().stream()
                .map(CommentDto::new)
                .toList();
    }
    @GetMapping("/find")
    public List<PostDto> getFilteredComments(@RequestParam("text") String text) {
        return commentService.findFilteredComments(text).stream()
                .map(PostDto::new)
                .toList();
    }


    @PostMapping
    public CommentDto createComment(@RequestParam("text") String text, @RequestParam("ownerId") Long id, @RequestParam("postId") Long postId){
        final Comment comment = commentService.addComment(text, id, postId);
        return new CommentDto(comment);
    }

    @PutMapping("/{id}")
    public CommentDto updateComment(@RequestParam("text") String text, @PathVariable Long id) {
        return new CommentDto(commentService.updateComment(id, text));
    }

    @DeleteMapping("/{id}")
    public CommentDto deleteComment(@PathVariable Long id) {
        return new CommentDto(commentService.deleteComment(id));
    }

    @DeleteMapping
    public void deleteAllComments(){
        commentService.deleteAllComments();
    }
}
