package com.webproglabs.lab1.lab34.controller;

import com.webproglabs.lab1.WebConfiguration;
import com.webproglabs.lab1.lab34.services.PostService;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/post")
public class PostController {
    private final PostService postService;

    public PostController(PostService postService) {
        this.postService = postService;
    }

    @GetMapping("/{id}")
    public PostDto getPost(@PathVariable Long id) {
        return new PostDto(postService.findPost(id));
    }

    @GetMapping
    public List<PostDto> getPosts() {
        return postService.findAllPosts().stream()
                .map(PostDto::new)
                .toList();
    }

    @GetMapping("/find")
    public List<PostDto> getFilteredPosts(@RequestParam("text") String text) {
        return postService.findFilteredPosts(text).stream()
                .map(PostDto::new)
                .toList();
    }

    @PostMapping
    public PostDto createPost(
            @RequestParam("text") String text,
            @RequestParam("authorId") Long authorId
    )
    {
        return new PostDto(postService.addPost(text, new ArrayList<>(),authorId));
    }

    @PutMapping("/{id}")
    public PostDto updatePost(
            @PathVariable Long id,
            @RequestParam("text") String text
    )
    {
        return new PostDto(postService.updatePost(id, text));
    }

    @DeleteMapping("/{id}")
    public PostDto deletePost (@PathVariable Long id) {
        return  new PostDto(postService.deletePost(id));
    }
    @DeleteMapping
    public void deleteAllPosts() {
        postService.deleteAllPosts();
    }
}
