package com.webproglabs.lab1.lab34.controller;

import com.webproglabs.lab1.WebConfiguration;
import com.webproglabs.lab1.lab34.model.Profile;
import com.webproglabs.lab1.lab34.model.UserRole;
import com.webproglabs.lab1.lab34.services.ProfileService;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/profile")
public class ProfileController {

    public static final String URL_LOGIN = "/jwt/login";

    private final ProfileService profileService;

    public ProfileController(ProfileService profileService) {
        this.profileService = profileService;
    }

    @GetMapping("/{id}")
    public ProfileDto getProfile(@PathVariable Long id) {
        return new ProfileDto(profileService.findUser(id));
    }

    @GetMapping("/find/{login}")
    public ProfileDto getProfileByLogin(@PathVariable String login) {
        return new ProfileDto(profileService.findByLogin(login));
    }

    @GetMapping
    public List<ProfileDto> getProfiles() {
        return profileService.findAllUsers().stream()
                .map(ProfileDto::new)
                .toList();
    }

    @PostMapping
    public ProfileDto createProfile(
            @RequestParam("login") String login,
            @RequestParam("password") String password
    )
    {
        return new ProfileDto(profileService.addUser(login, password, new ArrayList<>(),new ArrayList<>() ));
    }

    @PutMapping("/{id}")
    public ProfileDto updateProfile(
            @PathVariable Long id,
            @RequestParam("login") String login,
            @RequestParam("password") String password
    )
    {
        return new ProfileDto(profileService.updateUser(id, login, password));
    }

    @DeleteMapping("/{id}")
    public ProfileDto deleteProfile (@PathVariable Long id) {
        return  new ProfileDto(profileService.deleteUser(id));
    }
    @DeleteMapping
    public void deleteAllProfiles() {
        profileService.deleteAllUsers();
    }
    @PostMapping(URL_LOGIN)
    public String login(@RequestBody @Valid ProfileDto userDto) {
        return profileService.loginAndGetToken(userDto);
    }


    // этот метод юзать для проверки на администраторсткую роль вместо старой проверки по логину админа ЛОЛ
    //
    @GetMapping("role/{token}")
    public String getRoleByToken(@PathVariable String token) {
        var userDetails = profileService.loadUserByToken(token);
        Profile user = profileService.findByLogin(userDetails.getUsername());
        if (user != null) {
            return user.getRole().toString();
        }
        return null;
    }


}
