package com.webproglabs.lab1.lab34.controller.mvc_controllers;

import com.webproglabs.lab1.lab34.controller.CommentDto;
import com.webproglabs.lab1.lab34.controller.PostDto;
import com.webproglabs.lab1.lab34.controller.ProfileDto;
import com.webproglabs.lab1.lab34.model.Profile;
import com.webproglabs.lab1.lab34.services.CommentService;
import com.webproglabs.lab1.lab34.services.PostService;
import com.webproglabs.lab1.lab34.services.ProfileService;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;

@Controller
@RequestMapping("/feed")
public class FeedMvcController {
    private final ProfileService profileService;
    private final PostService postService;
    private final CommentService commentService;
    public FeedMvcController(ProfileService profileService, PostService postService, CommentService commentService) {
        this.profileService = profileService;
        this.postService = postService;
        this.commentService = commentService;
    }

    @GetMapping
    public String getFeedPage(Model model) {
        UserDetails principal = (UserDetails)SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        var user = profileService.findByLogin(principal.getUsername());
        if (user != null) {
            return "redirect:/feed/" + user.getId().toString();
        }

        model.addAttribute("profiles", profileService.findAllUsers().stream().map(ProfileDto::new).toList());
        return "feed";
    }

    @GetMapping(value = {"/{id}"})
    public String getFeedPageAuthorized(@PathVariable Long id, Model model) {
        model.addAttribute("profiles", profileService.findAllUsers().stream().map(ProfileDto::new).toList());
        model.addAttribute("posts", postService.findAllPosts().stream().map(PostDto::new).toList());
        model.addAttribute("selectedProfile", new ProfileDto(profileService.findUser(id)));

        return "feedPosts";
    }

    @GetMapping(value= {"/filter/{id}/"})
    public String getFeedPageFiltered(@PathVariable Long id, @RequestParam(value="searchField") String searchField, Model model) {
        model.addAttribute("profiles", profileService.findAllUsers().stream().map(ProfileDto::new).toList());
        model.addAttribute("posts", postService.findFilteredPosts(searchField).stream().map(PostDto::new).toList());
        model.addAttribute("selectedProfile", new ProfileDto(profileService.findUser(id)));
        return "feedPosts";
    }

    @PostMapping(value={"/post/{id}/"})
    public String createPost(@PathVariable Long id, @RequestParam(value="postInputField") String postInputField) {
        postService.addPost(postInputField, new ArrayList<>(), id);
        return "redirect:/feed/" + id.toString();
    }

    @PostMapping(value = {"/deletePost/{id}/{authorId}"})
    public String deletePost(@PathVariable Long id, @PathVariable Long authorId) {
        postService.deletePost(id);
        return "redirect:/feed/" + authorId.toString();
    }

    @GetMapping(value = {"postModal/{id}/{authorId}"})
    public String getPostEditModal(@PathVariable Long id,@PathVariable Long authorId, Model model) {
        model.addAttribute("selectedPost", new PostDto(postService.findPost(id)));
        model.addAttribute("profiles", profileService.findAllUsers().stream().map(ProfileDto::new).toList());
        model.addAttribute("posts", postService.findAllPosts().stream().map(PostDto::new).toList());
        model.addAttribute("selectedProfile", new ProfileDto(profileService.findUser(authorId)));
        return "editPostModal";
    }

    @PostMapping(value = {"editPost/{id}/{authorId}/"})
    public String editPost(@PathVariable Long id, @PathVariable Long authorId, @RequestParam(value="postEditField") String postEditField) {
        postService.updatePost(id, postEditField);
        return "redirect:/feed/" + authorId.toString();
    }

    @GetMapping(value = {"commentModal/{authorId}/{postId}"})
    public String getCommentModal(@PathVariable Long authorId,@PathVariable Long postId, Model model) {
        model.addAttribute("selectedPost", new PostDto(postService.findPost(postId)));
        model.addAttribute("profiles", profileService.findAllUsers().stream().map(ProfileDto::new).toList());
        model.addAttribute("posts", postService.findAllPosts().stream().map(PostDto::new).toList());
        model.addAttribute("selectedProfile", new ProfileDto(profileService.findUser(authorId)));
        return "commentModal";
    }

    @PostMapping(value = {"comment/{authorId}/{postId}/"})
    public String createComment(@PathVariable Long authorId,@PathVariable Long postId, @RequestParam(value="commentInputField") String commentInputField) {
        commentService.addComment(commentInputField, authorId, postId);
        return "redirect:/feed/" + authorId.toString();
    }

    @PostMapping(value = {"/deleteComment/{id}/{authorId}"})
    public String deleteComment(@PathVariable Long id, @PathVariable Long authorId) {
        commentService.deleteComment(id);
        return "redirect:/feed/" + authorId.toString();
    }

    @GetMapping(value = {"commentEditModal/{id}/{authorId}"})
    public String getCommentEditModal(@PathVariable Long id,@PathVariable Long authorId, Model model) {
        model.addAttribute("selectedComment", new CommentDto(commentService.findComment(id)));
        model.addAttribute("profiles", profileService.findAllUsers().stream().map(ProfileDto::new).toList());
        model.addAttribute("posts", postService.findAllPosts().stream().map(PostDto::new).toList());
        model.addAttribute("selectedProfile", new ProfileDto(profileService.findUser(authorId)));
        return "editCommentModal";
    }

    @PostMapping(value = {"editComment/{authorId}/{commentId}/"})
    public String editComment(@PathVariable Long authorId,@PathVariable Long commentId, @RequestParam(value="commentEditField") String commentEditField) {
        commentService.updateComment(commentId, commentEditField);
        return "redirect:/feed/" + authorId.toString();
    }
}
