package com.webproglabs.lab1.lab34.controller.mvc_controllers;

import com.webproglabs.lab1.lab34.controller.ProfileDto;
import com.webproglabs.lab1.lab34.model.UserRole;
import com.webproglabs.lab1.lab34.services.ProfileService;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;

@Controller
@RequestMapping("/")
public class ProfileMvcController {
    private final ProfileService profileService;

    public ProfileMvcController(ProfileService profileService) {
        this.profileService = profileService;
    }

    @GetMapping
    public String index() {
        return "default";
    }

    @GetMapping(value={"profiles"})
    @Secured({UserRole.AsString.ADMIN})
    public String getProfiles(Model model) {
        model.addAttribute("profiles", profileService.findAllUsers().stream().map(ProfileDto::new).toList());
        model.addAttribute("profileDto", new ProfileDto());
        return "profiles";
    }

    @GetMapping(value = {"profile/{login}"})
    public String getProfile(@PathVariable String login, Model model) {
        model.addAttribute("profile", new ProfileDto(profileService.findUserByLogin(login)));
        return "profilePage";
    }

    @PostMapping(value = {"profile/{id}"})
    public String deleteProfile(@PathVariable Long id) {
        profileService.deleteUser(id);
        return "redirect:/profiles";
    }

    @PostMapping(value = {"profile/create/"})
    public String createProfile(@ModelAttribute ProfileDto profileDto) {
        profileService.addUser(profileDto.getLogin(), profileDto.getPassword(), new ArrayList<>(), new ArrayList<>());
        return "redirect:/profiles";
    }
}
