package com.webproglabs.lab1.lab34.model;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Entity
public class Post {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    private String text;

    @OneToMany(mappedBy = "post", orphanRemoval = true, fetch = FetchType.EAGER)
    private List<Comment> comments = new ArrayList<Comment>();

    @ManyToOne()
    private Profile author;

    public Post(){}
    public Post(String text, Profile author, List<Comment> comments) {
        this.text = text;
        this.author = author;
        for (int i = 0; i < comments.size(); i++) {
            addComment(comments.get(i));
        }
    }

    public Long getId() {
        return id;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public Profile getAuthor() {
        return author;
    }

    public void setAuthor(Profile author) {
        this.author.getPosts().remove(this);
        this.author = author;
        if (!author.getPosts().contains(this)) {
            author.getPosts().add(this);
        }
    }

    public List<Comment> getComments() { return comments; }
    public int getCommentsSize() {
        if (comments == null) return 0;
        else return comments.size();
    }

    public void addComment(Comment comment) {
        this.comments.add(comment);
        if (comment.getPost() != this) {
            comment.setPost(this);
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Post post = (Post) o;
        return Objects.equals(id, post.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    public String ToString() {
        return "Post{" +
                "id=" + id +
                ", text='" + text + '\'' +
                ", author='" + author.ToString() + '\'' +
                ", comments='" + getCommentsSize() + '\'' +
                '}';
    }

}
