package com.webproglabs.lab1.lab34.services;

import com.webproglabs.lab1.lab34.model.Comment;
import com.webproglabs.lab1.lab34.model.Post;
import com.webproglabs.lab1.lab34.model.Profile;
import com.webproglabs.lab1.lab34.repository.CommentRepository;
import com.webproglabs.lab1.lab34.repository.PostRepository;
import com.webproglabs.lab1.lab34.repository.ProfileRepository;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityNotFoundException;
import javax.transaction.Transactional;
import java.util.List;
import java.util.Optional;

@Service
public class CommentService {

    private final ProfileRepository profileRepository;
    private final CommentRepository commentRepository;

    private final PostRepository  postRepository;

    public CommentService(ProfileRepository profileRepository, CommentRepository commentRepository, PostRepository postRepository) {
        this.profileRepository = profileRepository;
        this.commentRepository = commentRepository;
        this.postRepository = postRepository;
    }

    @Transactional
    public Comment findComment(Long id) {
        final Optional<Comment> comment = commentRepository.findById(id);
        return comment.orElseThrow(EntityNotFoundException::new);
    }

    @Transactional
    public List<Comment> findAllComments() {
        return commentRepository.findAll();
    }

    @Transactional
    public List<Post> findFilteredComments(String filter) {
        List<Post> postList = postRepository.findByTextLike("%" + filter + "%");
        List<Comment> commentList = commentRepository.findByTextLike("%" + filter + "%");
        List<Post> allPosts = postRepository.findAll();
        for(Post post : allPosts) {
            for (Comment comm : commentList) {
                if (post.getComments().contains(comm) && !(postList.contains(post))) {
                    postList.add(post);
                }
            }
        }
        return postList;
    }

    @Transactional
    public Comment addComment(String text, Long profileId, Long postId) {
        if (!StringUtils.hasText(text)) {
            throw new IllegalArgumentException("Comment data is null or empty");
        }
        try{
            Profile user = profileRepository.findById(profileId).get();
            Post post = postRepository.findById(postId).get();
            Comment comment = new Comment(text, user, post);
            user.addComment(comment);
            post.addComment(comment);
            profileRepository.save(user);
            postRepository.save(post);
            return commentRepository.save(comment);
        }
        catch (Exception exception) {
            return null;
        }
    }

    @Transactional
    public Comment updateComment(Long id, String text) {
        if (!StringUtils.hasText(text)) {
            throw new IllegalArgumentException("Comment data is null or empty");
        }
        Comment currentComment = findComment(id);
        currentComment.setText(text);
        final Profile owner = currentComment.getOwner();
        profileRepository.save(owner);
        final Post post = currentComment.getPost();
        postRepository.save(post);
        return commentRepository.save(currentComment);
    }

    @Transactional
    public Comment deleteComment(Long id) {
        final Comment currentComment = findComment(id);
        commentRepository.delete(currentComment);
        return currentComment;
    }
    @Transactional
    public void deleteAllComments() {
        commentRepository.deleteAll();
    }

}
