package com.webproglabs.lab1.lab34.services;

import com.webproglabs.lab1.lab34.model.Comment;
import com.webproglabs.lab1.lab34.model.Post;
import com.webproglabs.lab1.lab34.model.Profile;
import com.webproglabs.lab1.lab34.repository.CommentRepository;
import com.webproglabs.lab1.lab34.repository.PostRepository;
import com.webproglabs.lab1.lab34.repository.ProfileRepository;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityNotFoundException;
import javax.transaction.Transactional;
import java.util.List;
import java.util.Optional;

@Service
public class PostService {
    private final PostRepository postRepository;
    private final CommentRepository commentRepository;
    private final ProfileRepository profileRepository;

    public PostService(PostRepository postRepository, CommentRepository commentRepository, ProfileRepository profileRepository) {
        this.postRepository = postRepository;
        this.commentRepository = commentRepository;
        this.profileRepository = profileRepository;
    }

    @Transactional
    public Post findPost(Long id) {
        final Optional<Post> post = postRepository.findById(id);
        return post.orElseThrow(EntityNotFoundException::new);
    }

    @Transactional
    public List<Post> findAllPosts() {
        return postRepository.findAll();
    }

    @Transactional
    public List<Post> findFilteredPosts(String filter) {
        List<Post> postList = postRepository.findByTextLike("%" + filter + "%");
        List<Comment> commentList = commentRepository.findByTextLike("%" + filter + "%");
        List<Post> allPosts = postRepository.findAll();
        for(Post post : allPosts) {
            for (Comment comm : commentList) {
                if (post.getComments().contains(comm) && !(postList.contains(post))) {
                    postList.add(post);
                }
            }
        }
        return postList;
    }

    @Transactional
    public Post addPost (String text, List<Comment> comments, Long authorId) {
        if (!StringUtils.hasText(text)) {
            throw new IllegalArgumentException("Post data is null or empty");
        }
        Profile author = profileRepository.findById(authorId).get();
        Post post = new Post(text, author, comments);
        author.addPost(post);
        profileRepository.save(author);
        return postRepository.save(post);
    }

    @Transactional
    public Post updatePost(Long id, String text) {
        if (!StringUtils.hasText(text)) {
            throw new IllegalArgumentException("Post data is null or empty");
        }
        final Post currentPost = findPost(id);
        currentPost.setText(text);
        final Profile author = currentPost.getAuthor();
        profileRepository.save(author);
        return postRepository.save(currentPost);
    }

    @Transactional
    public Post deletePost(Long id) {
        final Post currentPost = findPost(id);
        postRepository.delete(currentPost);
        return currentPost;
    }

    @Transactional
    public void deleteAllPosts() {
        postRepository.deleteAll();
    }
}
